#include "chess.h"

//Zeit-Modi definieren
#define MODI_COUNT 3

static Game Games[MODI_COUNT] PROGMEM = { 
//Seconds,bonus,options,{guillotine1,2,...}
    {300, 0, 0, 0}, 
    {1500, 0, 0, 0},
    {0,0,GAME_OPTION_MANUAL,0}
    };


//Zhlvariablen initialisieren       
volatile uint8_t seconds[] = {0,0};
volatile uint8_t minutes[] = {0,0};
volatile uint8_t mseconds[] = {0,0};
volatile uint8_t updateDisplay = 0;
volatile uint8_t seconds10 = 0; 

unsigned long Timer = 0;
unsigned long Timer2 = 0;

//Andere Variablen
uint8_t Intensity = 20;

//Schach Variablen
uint8_t current = 0;
uint8_t pause = 1;
uint8_t game_mode = 1;

Game create_game(uint16_t time, uint16_t bonus)
{
    Game temp;
    temp.time = time;
    temp.bonus = bonus;
    return temp;   
}

void ChessInit(void)
{
    //Init MAX7219
    SPIMasterInit();
    SetMode(MAX_MODE_NORMAL);
    SetIntensity(Intensity / 10);
    SetDecodeMode(0x00);
    SetScanLimit(0x07);
    SetAll(0);
    
    //Inputs
    DDRD &= ~((1 << 6) | (1 << 7));
    //Enalbe Pull-Up-Resistor
    PORTD |= (1 << 6) | (1 << 7);
        
    //Init ADC
    ADCInit();
    
    //8bit Timer Init
    TCCR0 = (1<<CS00);          //Kein Teiler
    TIMSK |= (1<<TOIE0);        // Overflow Interrupt erlauben
    sei();                      // Global Interrupts aktivieren
}

void PlayManual(Game player1, Game player2)
{
    SetTime(0,player1.time);
    SetTime(1,player2.time);
    //KeyTest();
    updateDisplay = 1; 
    Timer = 0;
    while(1) 
    {    
        uint8_t key = GetKey();
        if(key != 255)
        SetDigit(0,chars[key]);
        switch(key)
        {
            case 4:
                if(Intensity < 0x0F * 10)
                { 
                    Intensity++;   
                    SetIntensity(Intensity / 10);
                }
                break;
            case 5:
                if(pause == 1)
                    pause = 2;
                if(pause == 0)
                    pause = 3;
                break;
            case 6:
                if(Intensity > 0x00)
                { 
                    Intensity--;   
                    SetIntensity(Intensity / 10);
                }
                break;
            case 9:
                pause = 1;
                return;
                break;
        };
        if(pause == 2 && key == 0)
            pause = 0;
        if(pause == 3 && key == 0)
            pause = 1;
        if(PIND & ((1 << 6) & (1 << 7)))
            current = 1 - current;
        else
        {
            if(PIND & (1 << 6))
                current = 1;
            if(PIND & (1 << 7))
                current = 0;
        }
        if(updateDisplay == 1 ) 
        {
            // die aktuelle Uhrzeit holen. Dabei aber dafr sorgen
            // dass sich die nicht schnell mal ndern kann
            cli();
            updateDisplay = 0;

            // ab jetzt haben wir alle Zeit der Welt die Zeit auf
            // die Anzeige zu bringen
            for(uint8_t i = 0; i != 2; i++)
            {
                Show2Digits(seconds[i],2 + 4*i);
                Show2DigitsDots(minutes[i],0 + 4*i,2);
            }
            sei();
        }
    }
}

void SetTime(uint8_t player, uint16_t time)
{
    minutes[player] = time / 60;
    seconds[player] = time - minutes[player] * 60;   
}

ISR (TIMER0_OVF_vect)
{
    if(!pause)
    {
        Timer += 256;
        if (Timer >= F_CPU / 10)
        {
            Timer -= F_CPU / 10;
            mseconds[current]--; 
            updateDisplay = 1;
            if(mseconds[current] == 255)
            {
                mseconds[current] = 9;
                seconds[current]--;                      
                if( seconds[current] == 255 ) 
                {
                    seconds[current] = 59;
                    minutes[current]--;
                    if( minutes[current] == 255 ) 
                    {
                        minutes[current] = 59;
                        Lost(current);
                    }
                }
            }
            updateDisplay = 1;
        }
    }
    Timer2 += 256;
    if (Timer2 >= F_CPU / 10)
    {
        Timer2 -= F_CPU / 10;
        seconds10++;   
    }
}

void Lost(uint8_t player)
{
    uint8_t screen = 1;
    while(1)
    {
        for(int i = 0; i != 4; i++)
            SetDigit(i + player*4, screen);
        screen = 1 - screen;
        _delay_ms(500);
    }
}

void MainMenu(void)
{
    Game temp;
    SetAll(0);
    uint8_t key = 255;
    while(1)
    {
        Show2Digits(game_mode,0);
        key = GetKey();
        seconds10 = 0;
             
        switch(key)
        {
            case 6: 
                if(game_mode > 1)
                    game_mode--; 
                 break;
            case 5:
                temp = GetGame(game_mode);
                PlayManual(temp,temp);
                break;
            case 4:
                if(game_mode < MODI_COUNT)
                    game_mode++;
                break;
        };
    }
}

Game GetGame(uint8_t modi)
{
    Game temp;
    temp.time = pgm_read_word (&(Games[modi].time));
    temp.bonus = pgm_read_word (&(Games[modi].bonus));
    temp.options = pgm_read_byte(&(Games[modi].options));
    /*for(((temp.options << 5) & 0x1F) > 0)
    {
        uint16_t *ptr = (uint16_t*) pgm_read_word(&(Games[modi].guillotine));
        temp.guillotine = (uint16_t*) pgm_read_word(ptr);
    }*/
    uint16_t *ptr = (uint16_t*)pgm_read_word(&(Games[modi].guillotine));
    uint16_t value[7];
    for(int i = 0; i<((temp.options << 5) & 224);i++)
    {
        value[i] = pgm_read_word(ptr + 2*i);
        if(i == 0)
             temp.guillotine = (uint16_t*)&value;
    }
    return temp;
}
